"""
Usage Tutorial: Automated Backup Script with Encryption

This script allows you to back up directories to OpticalBackup with encryption using a user-provided key.

---

### How to Configure:

1. **Edit Backup Directories**:
   - Locate the `BACKUP_DIRS` variable in the script.
   - Add the directories you want to back up as a list of paths:
     BACKUP_DIRS = ["C:\\path\\to\\dir1", "C:\\path\\to\\dir2"]
   - Ensure all directories exist on your system.

2. **Set Your API Token**:
   - Obtain your API token from the OpticalBackup dashboard: [API Tokens](https://app.opticalbackup.com/api-tokens).
   - Replace the placeholder in the `AUTH_TOKEN` variable with your token:
     AUTH_TOKEN = "YOUR_API_TOKEN_HERE"

3. **Provide Your Encryption Key**:
   - Create a file containing your encryption key (e.g., `my_key.pem`).
   - Save the file in the same directory as the script or specify the full path in the `USER_KEY_FILE` variable:
     USER_KEY_FILE = "my_key.pem"

---

### How to Run:

1. Save the script as `backup_script.py`.
2. Ensure dependencies are installed:
   - Install Python modules:
     pip install requests
   - Install OpenSSL for encryption:
     Download and install OpenSSL for Windows: https://slproweb.com/products/Win32OpenSSL.html
   - Add the OpenSSL binary to your system PATH or specify its path in the script.
3. Execute the script:
   python backup_script.py

---
"""

import os
import requests
import subprocess
import time
from pathlib import Path

# Configuration
BACKUP_DIRS = [r"C:\\path\\to\\dir1"]  # Directories to back up
AUTH_TOKEN = "YOUR_API_TOKEN_HERE"  # User API token
USER_KEY_FILE = "my_key.pem"  # User-provided encryption key file

API_URL = "https://app.opticalbackup.com/api"  # API endpoint. Don't edit
REPORT_FILE = f"backup_report_{time.strftime('%Y%m%d%H%M%S')}.log"
CONTAINER_NAME = f"Backup Container {time.strftime('%Y-%m-%d %H:%M:%S')}"
CONTAINER_DESCRIPTION = f"Automated backup on {time.strftime('%Y-%m-%d')}"
UPLOAD_LIMIT = 0  # To be fetched dynamically

# Check if the user encryption key exists
if not Path(USER_KEY_FILE).is_file():
    raise FileNotFoundError(f"User encryption key file '{USER_KEY_FILE}' not found.")

with open(USER_KEY_FILE, "r") as key_file:
    USER_KEY = key_file.read().strip()

if not USER_KEY:
    raise ValueError("The provided encryption key file is empty.")

# Fetch the upload limit from the server
def fetch_upload_limit():
    global UPLOAD_LIMIT
    print("Fetching file upload limit...")
    response = requests.get(
        f"{API_URL}/client",
        headers={"Authorization": f"Bearer {AUTH_TOKEN}", "Accept": "application/json"},
    )
    if response.status_code != 200:
        raise RuntimeError(f"Failed to fetch upload limit: {response.text}")
    UPLOAD_LIMIT = response.json().get("file_upload_limit", 0)
    print(f"File upload limit: {UPLOAD_LIMIT} bytes")

# Create a new container
def create_container():
    print("Creating container...")
    response = requests.post(
        f"{API_URL}/containers",
        headers={"Authorization": f"Bearer {AUTH_TOKEN}", "Accept": "application/json"},
        data={"name": CONTAINER_NAME, "description": CONTAINER_DESCRIPTION},
    )
    if response.status_code != 200:
        raise RuntimeError(f"Failed to create container: {response.text}")
    container_id = response.json().get("id")
    if not container_id:
        raise RuntimeError(f"Container creation failed: {response.text}")
    print(f"Container created with ID: {container_id}")
    return container_id

# Encrypt a file with the user-provided key
def encrypt_file_with_user_key(file_path):
    encrypted_file = f"{file_path}.gpg"
    openssl_binary = "C:\\Path\\to\\openssl.exe"  # Replace with the actual path to OpenSSL binary
    command = [
        openssl_binary,
        "enc",
        "-aes-256-cbc",
        "-salt",
        "-pbkdf2",
        "-pass",
        f"pass:{USER_KEY}",
        "-in",
        str(file_path),
        "-out",
        encrypted_file,
    ]
    result = subprocess.run(command, capture_output=True)
    if result.returncode != 0:
        raise RuntimeError(f"Error encrypting file {file_path}: {result.stderr.decode()}")
    return encrypted_file

# Upload files to the container
def upload_files(container_id):
    for directory in BACKUP_DIRS:
        dir_path = Path(directory)
        if not dir_path.is_dir():
            print(f"Directory {dir_path} does not exist. Skipping.")
            continue

        print(f"Backing up directory: {dir_path}")
        for file_path in dir_path.rglob("*"):
            if not file_path.is_file():
                continue

            file_size = file_path.stat().st_size
            if file_size > UPLOAD_LIMIT:
                print(f"Skipping file: {file_path} (Size: {file_size} bytes exceeds limit: {UPLOAD_LIMIT} bytes)")
                with open(REPORT_FILE, "a") as report:
                    report.write(f"Skipped file {file_path} (Size: {file_size} bytes exceeds limit: {UPLOAD_LIMIT} bytes)\n")
                continue

            print(f"Encrypting file: {file_path}")
            encrypted_file = encrypt_file_with_user_key(file_path)
            print(f"Uploading encrypted file: {encrypted_file}")
            with open(encrypted_file, "rb") as enc_file:
                response = requests.post(
                    f"{API_URL}/containers/{container_id}/upload",
                    headers={"Authorization": f"Bearer {AUTH_TOKEN}", "Accept": "application/json"},
                    files={"files[]": enc_file},
                    data={"encrypted": "1", "obpubkey": "0", "paths[]": str(file_path.parent)},
                )
                with open(REPORT_FILE, "a") as report:
                    report.write(f"Response for {file_path}: {response.text}\n")
            os.remove(encrypted_file)

# Close the container
def close_container(container_id):
    print("Closing container...")
    response = requests.patch(
        f"{API_URL}/containers/{container_id}/close",
        headers={"Authorization": f"Bearer {AUTH_TOKEN}", "Accept": "application/json"},
    )
    with open(REPORT_FILE, "a") as report:
        report.write(f"Close container response: {response.text}\n")
    print("Container closed.")

# Main execution
if __name__ == "__main__":
    with open(REPORT_FILE, "w") as report:
        report.write(f"Backup Report - {time.strftime('%Y-%m-%d %H:%M:%S')}\n")
        report.write(f"Container Name: {CONTAINER_NAME}\n")
        report.write(f"Container Description: {CONTAINER_DESCRIPTION}\n")
        report.write("-" * 50 + "\n")

    try:
        fetch_upload_limit()
        container_id = create_container()
        upload_files(container_id)
        close_container(container_id)
        print(f"Backup completed. Report saved to {REPORT_FILE}")
    except Exception as e:
        print(f"Error: {e}")
