import os
import requests
import logging
from pathlib import Path

"""
Usage Tutorial: Automated Backup Script with Logging

This script automates the process of backing up directories to OpticalBackup, with detailed logging and API integration.

---

### How to Configure:

1. **Edit Backup Directories**:
   - Locate the `BACKUP_DIRS` variable in the script.
   - Add the directories you want to back up as a list of paths:
     BACKUP_DIRS = ["/path/to/dir1", "/path/to/dir2"]
   - Ensure all directories exist on your system.

2. **Set Your API Token**:
   - Obtain your API token from the OpticalBackup dashboard: [API Tokens](https://app.opticalbackup.com/api-tokens).
   - Replace the placeholder in the `AUTH_TOKEN` variable with your token:
     AUTH_TOKEN = "YOUR-API-KEY"

3. **Review Configuration**:
   - API URL: Ensure `API_URL` is set to the correct endpoint for OpticalBackup:
     API_URL = "https://ob.nunomicaelo.pt/api"
   - Container Details: Customize `CONTAINER_NAME` and `CONTAINER_DESCRIPTION` if needed.
   - Log File: Logs are saved to `backup_report_<script_name>.log`.

---

### How to Run:

1. Save the script as `backup_script.py`.
2. Ensure dependencies are installed:
   - Install Python modules:
     pip install requests
3. Execute the script:
   python3 backup_script.py

---

### Script Features:

1. **Customizable Backup Directories**:
   - Specify multiple directories to include in the backup.

2. **File Upload Management**:
   - Automatically skips files larger than the upload limit retrieved from the API.

3. **API Integration**:
   - Creates a container, uploads files, and closes the container using OpticalBackup APIs.

4. **Detailed Logging**:
   - Logs detailed information and errors to `backup_report_<script_name>.log`.

5. **Error Handling**:
   - Provides clear error messages for issues during execution.

---

For more help or troubleshooting, refer to the documentation or contact OpticalBackup support at micaelo@opticalbackup.com.
"""

# Add root directories to backup ["/path/to/dir1", "/path/to/dir2"]
BACKUP_DIRS = ["/path/to/dir1", "/path/to/dir2"]

# API key. Get yours here: https://app.opticalbackup.com/api-tokens
AUTH_TOKEN = "YOUR-API-KEY"

# Configuration
API_URL = "https://ob.nunomicaelo.pt/api"
CONTAINER_NAME = f"Backup Container {Path(__file__).stem}"
CONTAINER_DESCRIPTION = f"Automated backup on {Path(__file__).stem}"
LOG_FILE = f"backup_report_{Path(__file__).stem}.log"

# Set up logging
logging.basicConfig(filename=LOG_FILE, level=logging.INFO, format="%(asctime)s - %(message)s")

# Functions
def fetch_upload_limit():
    """Fetch the maximum upload limit from the server."""
    response = requests.get(f"{API_URL}/client", headers={"Authorization": f"Bearer {AUTH_TOKEN}"})
    response.raise_for_status()
    upload_limit = response.json().get("file_upload_limit")
    if upload_limit is None:
        raise ValueError("Failed to retrieve upload limit.")
    logging.info(f"File upload limit: {upload_limit} bytes")
    return upload_limit

def create_container():
    """Create a new backup container."""
    data = {"name": CONTAINER_NAME, "description": CONTAINER_DESCRIPTION}
    response = requests.post(f"{API_URL}/containers", headers={"Authorization": f"Bearer {AUTH_TOKEN}"}, data=data)
    response.raise_for_status()
    container_id = response.json().get("id")
    if not container_id:
        raise ValueError("Failed to create container.")
    logging.info(f"Created container with ID: {container_id}")
    return container_id

def upload_file(container_id, file_path, dir_path):
    """Upload a file to the container."""
    print(f"Uploading file: {file_path} to directory: {dir_path}")  # Added for verbosity
    with open(file_path, "rb") as file:
        files = {"files[]": file}
        data = {"paths[]": dir_path}
        response = requests.post(
            f"{API_URL}/containers/{container_id}/upload",
            headers={"Authorization": f"Bearer {AUTH_TOKEN}"},
            files=files,
            data=data,
        )
        response.raise_for_status()
        logging.info(f"Uploaded {file_path}")
        
def close_container(container_id):
    """Close the container."""
    response = requests.patch(
        f"{API_URL}/containers/{container_id}/close", headers={"Authorization": f"Bearer {AUTH_TOKEN}"}
    )
    response.raise_for_status()
    logging.info(f"Closed container ID: {container_id}")

def backup_directories(container_id, upload_limit):
    """Backup specified directories."""
    for backup_dir in BACKUP_DIRS:
        if os.path.isdir(backup_dir):
            for root, _, files in os.walk(backup_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    file_size = os.path.getsize(file_path)
                    if file_size > upload_limit:
                        logging.warning(f"Skipped {file_path} (Size: {file_size} exceeds limit: {upload_limit})")
                    else:
                        upload_file(container_id, file_path, root)
        else:
            logging.warning(f"Directory {backup_dir} does not exist. Skipping.")

# Main Script
if __name__ == "__main__":
    try:
        upload_limit = fetch_upload_limit()
        container_id = create_container()
        backup_directories(container_id, upload_limit)
        close_container(container_id)
        print(f"Backup completed. Logs are saved in {LOG_FILE}")
    except Exception as e:
        logging.error(f"Error: {e}")
        print(f"An error occurred. Check {LOG_FILE} for details.")
