#!/bin/bash
#
# Usage Tutorial: Automated Backup Script for Linux
#
# This script allows you to back up directories to OpticalBackup, handling file uploads efficiently with optional size limits.
#
# ---
#
# ### How to Configure:
#
# 1. **Edit Backup Directories**:
#    - Locate the `BACKUP_DIRS` variable in the script.
#    - Add the directories you want to back up as a space-separated list of paths:
#      BACKUP_DIRS=("/path/to/dir1" "/path/to/dir2")
#    - Ensure all directories exist on your system.
#
# 2. **Set Your API Token**:
#    - Obtain your API token from the OpticalBackup dashboard: https://app.opticalbackup.com/api-tokens
#    - Replace the placeholder in the `AUTH_TOKEN` variable with your token:
#      AUTH_TOKEN="YOUR_API_TOKEN_HERE"
#
# ---
#
# ### How to Run:
#
# 1. Save the script as `backup_script.sh`.
# 2. Ensure dependencies are installed:
#    - Install `curl` for API requests (most systems include it by default).
#    - Install `jq` for JSON parsing:
#      sudo apt install jq       # For Ubuntu/Debian
#      sudo dnf install jq       # For Fedora
# 3. Make the script executable:
#    chmod +x backup_script.sh
# 4. Execute the script:
#    ./backup_script.sh
#
# ---
#
# ### Script Features:
#
# 1. **Customizable Backup Directories**:
#    - Add multiple directories to `BACKUP_DIRS`.
#
# 2. **API Integration**:
#    - Automatically creates a container, uploads files, and closes the container.
#
# 3. **File Size Limit Handling**:
#    - Skips files larger than the upload limit retrieved from the API.
#
# 4. **Detailed Logging**:
#    - Generates a report saved in the script's directory, named `backup_report_YYYYMMDDHHMMSS.log`.
#
# ---
#
# For more help or troubleshooting, refer to the documentation or contact OpticalBackup support
# at micaelo@opticalbackup.com.

# Add root directories to backup ("/path/to/dir1" "/path/to/dir2")
BACKUP_DIRS=("/path/to/dir1" "/path/to/dir2")

# API key. Get yours here: https://app.opticalbackup.com/api-tokens
AUTH_TOKEN="YOUR-API-KEY"

# Configuration
API_URL="https://ob.nunomicaelo.pt/api" #Don't edit

CONTAINER_NAME="Backup Container $(date +'%Y-%m-%d %H:%M:%S')"
CONTAINER_DESCRIPTION="Automated backup on $(date +'%Y-%m-%d')"

REPORT_FILE="backup_report_$(date +'%Y%m%d%H%M%S').log"
UPLOAD_LIMIT=0 # To be retrieved dynamically


# Function to fetch the upload limit
fetch_upload_limit() {
    echo "Fetching file upload limit..."
    RESPONSE=$(curl --silent --location "$API_URL/client" \
        --header "Accept: application/json" \
        --header "Authorization: Bearer $AUTH_TOKEN")
    UPLOAD_LIMIT=$(echo "$RESPONSE" | jq -r '.file_upload_limit')
    if [[ -z "$UPLOAD_LIMIT" || "$UPLOAD_LIMIT" == "null" ]]; then
        echo "Failed to fetch upload limit. Response: $RESPONSE"
        exit 1
    fi
    echo "File upload limit: $UPLOAD_LIMIT bytes"
}

# Function to create a container
create_container() {
    echo "Creating container..."
    RESPONSE=$(curl --silent --location "$API_URL/containers" \
        --header "Accept: application/json" \
        --header "Authorization: Bearer $AUTH_TOKEN" \
        --form "name=$CONTAINER_NAME" \
        --form "description=$CONTAINER_DESCRIPTION")
    CONTAINER_ID=$(echo "$RESPONSE" | jq -r '.id')
    if [[ -z "$CONTAINER_ID" || "$CONTAINER_ID" == "null" ]]; then
        echo "Failed to create container. Response: $RESPONSE"
        exit 1
    fi
    echo "Container created with ID: $CONTAINER_ID"
}

# Function to upload files
upload_files() {
    for DIR in "${BACKUP_DIRS[@]}"; do
        if [[ -d "$DIR" ]]; then
            echo "Backing up directory: $DIR"
            find "$DIR" -type f | while read -r FILE; do
                FILE_SIZE=$(stat --format="%s" "$FILE")
                DIR_PATH=$(dirname "$FILE") # Extract directory path without the file name
                if (( FILE_SIZE > UPLOAD_LIMIT )); then
                    echo "Skipping file: $FILE (Size: $FILE_SIZE bytes exceeds limit: $UPLOAD_LIMIT bytes)"
                    echo "Warning: Skipped file $FILE (Size: $FILE_SIZE bytes exceeds limit: $UPLOAD_LIMIT bytes)" >> "$REPORT_FILE"
                else
                    echo "Uploading file: $FILE"
                    RESPONSE=$(curl --silent --location "$API_URL/containers/$CONTAINER_ID/upload" \
                        --header "Accept: application/json" \
                        --header "Authorization: Bearer $AUTH_TOKEN" \
                        --form "files[]=@$FILE" \
                        --form "paths[]=$DIR_PATH")
                    echo "Response: $RESPONSE" >> "$REPORT_FILE"
                fi
            done
        else
            echo "Directory $DIR does not exist. Skipping."
        fi
    done
}
# Function to close the container
close_container() {
    echo "Closing container..."
    RESPONSE=$(curl --silent --location --request PATCH "$API_URL/containers/$CONTAINER_ID/close" \
        --header "Accept: application/json" \
        --header "Authorization: Bearer $AUTH_TOKEN")
    echo "Response: $RESPONSE" >> "$REPORT_FILE"
    echo "Container closed."
}

# Create report header
echo "Backup Report - $(date)" > "$REPORT_FILE"
echo "Container Name: $CONTAINER_NAME" >> "$REPORT_FILE"
echo "Container Description: $CONTAINER_DESCRIPTION" >> "$REPORT_FILE"
echo "------------------------------------------" >> "$REPORT_FILE"

# Execute backup steps
fetch_upload_limit
create_container
upload_files
close_container

# Final report summary
echo "Backup completed. Report saved to $REPORT_FILE."
